#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v1ll4n>
  Purpose: Test for decologger
  Created: 2016/12/18
"""

import unittest
import traceback
from decologger import Decologger


########################################################################
class DecologgerTest(unittest.case.TestCase):
    """"""

    #----------------------------------------------------------------------
    def test_learn_decorector(self):
        """"""
        print('='*64)
        def logit(func):
            def wrapper(*args, **kwargs):
                print('wrapper got param', args, kwargs)
                print('pre call', func.__name__)
                return func(*args, **kwargs)
            return wrapper
    
        #----------------------------------------------------------------------
        @logit
        def funct(args):
            """"""
            print('funct called! Got Param', args)
        
        funct('Hello')
        
        def sss(*arg):
            print arg
            print 'sssss called' 
            return logit(arg[0])
        
        @sss
        def tests():
            print 'tests called'
        
        tests()
        
        print('='*64)
    
    def test_exception(self):
        try:
            raise Exception('adfasdfasdf')
        except:
            print traceback.extract_stack()
            traceback.format_exc()
    
    #----------------------------------------------------------------------
    def test_decologger_basic_api(self):
        """"""
        dclogger = Decologger(name='testlogger')
        
        
        class A(object):
            #----------------------------------------------------------------------
            def __init__(self):
                """"""
                pass
            
            #----------------------------------------------------------------------
            @dclogger.crucial
            def B(self):
                """"""
                print 'B Called!'
        
        
        dclogger.critical('Hello Critical')
        A().B()
                
        
        print('='*64)
        
        
        
    
    

if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v1ll4n>
  Purpose: Parse dict
  Created: 2016/12/16
"""

import shelve
import unittest
import os
from pprint import pprint
from hashlib import md5

SESSION_TABLE_FILE = 'sessions.dat'
DEFAULT_SESSION_ID = 'default'

########################################################################
class DictParser():
    """"""

    #----------------------------------------------------------------------
    def __init__(self, filename, 
                 session_id=SESSION_TABLE_FILE,
                 do_continue=False,
                 session_data_file=SESSION_TABLE_FILE):
        """Constructor
        
        Params:
            filename: :str: the target dict you want to use
            session_id: :str: the id you want to use to identify your session
            do_continue: :bool: continue or not
            session_data_file: :str: the file you want to save session info
            """
        abspathfile = os.path.abspath(filename)
        if os.path.exists(abspathfile):
            self._filename = filename
        else:
            raise StandardError('[!] No Such Dict File')
        
        self._session_data_file = session_data_file
        self._session_id = md5(session_id+filename).hexdigest()
        
        self._dict_file_p = open(self._filename)
        self._session_progress_table = shelve.open(os.path.abspath(self._session_data_file))
        
        # continue last task
        if do_continue:
            if self._session_progress_table.has_key(self._session_id):
                pos = self._session_progress_table[self._session_id]
                self._dict_file_p.seek(pos)

    #----------------------------------------------------------------------
    def __iter__(self):
        """"""
        return self
        

    #----------------------------------------------------------------------
    def next(self):
        """"""
        return self._next()
    
    #----------------------------------------------------------------------
    def _next(self):
        """"""
        self._current_pos = 0
        self._current_pos = self._dict_file_p.tell()
        while True:
            ret = self._dict_file_p.readline()
            self.save()
            if ret.strip() == '':
                #_empty_lines_count = _empty_lines_count + 1
                if self._dict_file_p.tell() == self._current_pos:
                    raise StopIteration()
                else:
                    continue
            else:
                return ret.strip()
            
    #----------------------------------------------------------------------
    def save(self):
        """"""
        if self._current_pos - self._dict_file_p.tell() >= 1024:
            self._save()        
        #print self._session_progress_table
    
    
    #----------------------------------------------------------------------
    def _save(self):
        """Save the progress"""
        self._session_progress_table[self._session_id] = self._dict_file_p.tell()
     
    #----------------------------------------------------------------------
    def force_save(self):
        """Force save"""
        self._save()
        
    
    #----------------------------------------------------------------------
    def __del__(self):
        """Close the opened resource"""
        try:
            self._dict_file_p.close()
            self._session_progress_table.close()
        except AttributeError:
            pass
 
    #----------------------------------------------------------------------
    def get_next_collection(self, num=200):
        """Returns a tuple for the next [num] paylaod(lines)"""
        ret = []
        for i in xrange(num):
            try:
                ret.append(self._next())
            except StopIteration:
                pass
        self._save()
        
        return tuple(ret)
    
    #----------------------------------------------------------------------
    def get_current_pos(self):
        """The current pos"""
        return self._dict_file_p.tell()
    
    #----------------------------------------------------------------------
    def get_total_size(self):
        """total size(progress)"""
        return os.path.getsize(self._filename)
        
    #----------------------------------------------------------------------
    def get_fp(self):
        """Get the fp(Dangerous)"""
        return self._dict_file_p
        
        

if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v1ll4n>
  Purpose: Provide some useful thread utils
  Created: 2016/10/29
"""
import uuid
import time
import unittest
from Queue import Queue, Empty
import threading
from threading import Thread
import inspect
import traceback


#----------------------------------------------------------------------
def start_thread(func, *args, **kwargs):
    """"""
    ret = Thread(target=func, args=args, kwargs=kwargs)
    ret.daemon = True
    ret.start()


########################################################################
class Contractor(object):
    """Create Multi-Thread to support the 
    concurrence of many tasks"""

    #----------------------------------------------------------------------
    def __init__(self, thread_max=50):
        """Constructor"""
        self.task_list = []
        self.result_queue = Queue()

        self.lock = threading.Lock()

        self.thread_max = thread_max
        self._current_thread_count = 0
        
        self._executed_task_count = 0
        self._task_count = 0

    def _uuid1_str(self):
        '''Returns: random UUID tag '''
        return str(uuid.uuid1())

    #----------------------------------------------------------------------
    def feed(self, target_func, *vargs, **kwargs):
        """"""
        self.add_task(target_func, *vargs, **kwargs)
        
    def add_task(self, target_func, *args, **argv):
        '''Add task to Pool and wait to exec

        Params:
            target_func : A callable obj, the entity of the current task
            args : the args of [target_func]
            argv : the argv of [target_func]
        '''
        assert callable(target_func), '[!] Function can \'t be called'

        ret = {}
        ret['func'] = target_func
        ret['args'] = args
        ret['argv'] = argv
        #ret['uuid'] = self.signal_name
        self._task_count = self._task_count + 1
        self.task_list.append(ret)

    def start(self):
        """"""
        ret = Thread(target=self._run)
        ret.daemon = True
        ret.start()

        return self.result_queue

    #----------------------------------------------------------------------
    def _run(self):
        """"""
        for i in self.task_list:
            #print self.current_thread_count
            while self.thread_max <= self._current_thread_count:
                time.sleep(0.3)
            self._start_task(i)        

    def _start_task(self, task):
        """"""
        self._current_thread_count = self._current_thread_count + 1
        try:

            ret = Thread(target=self._worker, args=(task,))
            ret.daemon = True
            ret.start()
        except TypeError:
            self._current_thread_count = self._current_thread_count - 1

    def _worker(self, dictobj):
        """"""
        func = dictobj['func']
        args = dictobj['args']
        argv = dictobj['argv']

        try:
            result = func(*args, **argv)
        except Exception, e:
            #print 'ecp occured'
            result = tuple([e, traceback.extract_stack()])
        
        self.lock.acquire()
        self._executed_task_count = self._executed_task_count + 1
        self._add_result_to_queue(result=result)
        self.lock.release()

    def _add_result_to_queue(self, **kw):
        """"""
        assert kw.has_key('result'), '[!] Result Error!'

        self.result_queue.put(kw['result']) 
        self._current_thread_count = self._current_thread_count - 1
        
    #----------------------------------------------------------------------
    def get_result_queue(self):
        """"""
        return self.result_queue

    #----------------------------------------------------------------------
    def get_task_list(self):
        """"""
        self.task_list
    
    #----------------------------------------------------------------------
    def get_result_generator(self):
        """"""
        while True:
            try:
                ret = self.result_queue.get(timeout=1)
                yield ret
            except Empty:
                if self._task_count == self._executed_task_count:
                    break
                else:
                    pass        
    
    #----------------------------------------------------------------------
    @property
    def task_count(self):
        """"""
        return self._task_count
    
    #----------------------------------------------------------------------
    @property
    def executed_task_count(self):
        """"""
        return self._executed_task_count
    
    #----------------------------------------------------------------------
    @property    
    def percent(self):
        """"""
        return float(self._task_count)/float(self._executed_task_count)
    
    #----------------------------------------------------------------------
    @property
    def current_thread_count(self):
        """"""
        return self._current_thread_count
        
        
        


class UtilsTest(unittest.case.TestCase):
    def runTest(self):
        ms = inspect.getmembers(self)
        ms = map(lambda x: x[0], ms)
        for i in ms:
            if callable(getattr(self,i)):
                if i.startswith('test_'):
                    getattr(self, i)()    

    def test_pool(self):
        def demo_task(*args):
            '''simulate the plugin.run'''
            print '[!] Computing!'
            time.sleep(args[0])
            print '[!] Finished!'
            print
            returns = 'Runtime Length : %s' % str(args)
            return returns
        pool = Contractor()
        pool.add_task(demo_task, 7)
        pool.add_task(demo_task, 3)
        q = pool.start()
        print pool._current_thread_count
        self.assertIsInstance(q, Queue)

        r = q.get()
        print r
        self.assertIsInstance(r, str)
        r = q.get()
        print r
        self.assertIsInstance(r, str)

        print pool._current_thread_count


if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v1ll4n>
  Purpose: Test the host is alive?
  Created: 2016/12/20
"""

import unittest
from ping import verbose_ping

#----------------------------------------------------------------------
def is_alive(host, timeout=2, count=4):
    """"""
    result = {}
    
    ret = ping_host(host, timeout, count)
    result['ping'] = ret
    
    return result


#----------------------------------------------------------------------
def ping_host(host, timeout=2, count=4):
    """"""
    result = {}
    result['alive'] = False
    result['delay'] = 0
    try:
        ping_delay = verbose_ping(host, timeout=timeout,count=count)
        if ping_delay != 0:
            result['alive'] = True
            result['delay'] = ping_delay
            return result
        else:
            raise ValueError()
    except:
        return result


########################################################################
class IsAliveTest(unittest.case.TestCase):
    """"""

    #----------------------------------------------------------------------
    def test_pingtest(self):
        """Ping test"""
        
        print ping_host('45.78.6.64')
        
        
    
    

if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v1ll4n>
  Purpose: G3ar Test
  Created: 2016/12/23
"""

import unittest

from time import sleep
from threadutils import thread_pool, contractor
from taskbulter import task_bulter
from Queue import Empty
ThreadPool = thread_pool.Pool
Contractor = contractor.Contractor
TaskBulter = task_bulter.TaskBulter

def tasktest(arg1):
    print(arg1)
    def runforever():
        while True:
            pass
    
    pool = ThreadPool()
    pool.start()
    pool.feed(runforever)
    pool.feed(runforever)
    pool.feed(runforever)
    while True:
        pass

########################################################################
class G3arThreadUtilsTest(unittest.case.TestCase):
    """"""

    #----------------------------------------------------------------------
    def test_threadpool(self):
        """"""
        
        def testthreadpool_function(arg1, arg2='adsf'):
            sleep(1)
            return arg1, arg2

        pool = ThreadPool(thread_max=30)

        pool.start()

        for i in range(40):
            pool.feed(testthreadpool_function, arg1=i)


        result_gen = pool.get_result_generator()
        for i in result_gen:
            print pool.task_count
            print pool.executed_task_count
            print pool.percent            
            print i


        for i in range(21,41):
            pool.feed(testthreadpool_function, arg1=i)


        result_queue = pool.get_result_queue()

        while True:
            try:
                print result_queue.get(timeout=2)
            except Empty:
                print 'End!'
                break
    
    #----------------------------------------------------------------------
    def test_contractor(self):
        """"""
        
        def testcontractor_function(arg1, arg2='adsf'):
            sleep(1)
            return arg1, arg2

        ctr = Contractor(thread_max=30)
    
        for i in range(40):
            ctr.feed(testcontractor_function, arg1=i)
        
        ctr.start()
    
        result_gen = ctr.get_result_generator()

            # Generator 的使用方法
        for i in result_gen:
            print i

        ctr = Contractor(thread_max=50)

        def testcontractor_function_ecp(arg1, arg2='adsf'):
            sleep(1)
            raise Exception('test exception')
            #return arg1, arg2
        
        for i in range(41, 81):
            ctr.feed(testcontractor_function_ecp, arg1=i)
    
        ctr.start()

            
        result_queue = ctr.get_result_queue()
    
        while True:
            try:
                print result_queue.get(timeout=2)
            except Empty:
                print 'End!'
                break
        
        


########################################################################
class G3arTaskBulterTest(unittest.case.TestCase):
    """"""

    #----------------------------------------------------------------------
    def test_taskbulter(self):
        """Constructor"""
        bulter = TaskBulter(threads_update_interval=0.1)
            
        bulter.start_task(id='tasktest', target=tasktest, args=(5,))
        task = bulter.get_task_by_id('tasktest')
        print task
        sleep(2)
        print bulter.get_tasks_status()
        bulter.destory_task(task)
        #bulter.close()
    
    
if __name__ == '__main__':
    unittest.main()



# Copyright (c) 2013, 2014, 2015, 2016 Philip Hane
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import sys
import re
import copy
from datetime import datetime
import logging
from .utils import unique_everseen
from . import (BlacklistError, WhoisLookupError, NetError)

if sys.version_info >= (3, 3):  # pragma: no cover
    from ipaddress import (ip_address,
                           ip_network,
                           summarize_address_range,
                           collapse_addresses)
else:  # pragma: no cover
    from ipaddr import (IPAddress as ip_address,
                        IPNetwork as ip_network,
                        summarize_address_range,
                        collapse_address_list as collapse_addresses)

log = logging.getLogger(__name__)

# Legacy base whois output dictionary.
BASE_NET = {
    'cidr': None,
    'name': None,
    'handle': None,
    'range': None,
    'description': None,
    'country': None,
    'state': None,
    'city': None,
    'address': None,
    'postal_code': None,
    'emails': None,
    'created': None,
    'updated': None
}

RIR_WHOIS = {
    'arin': {
        'server': 'whois.arin.net',
        'fields': {
            'name': r'(NetName):[^\S\n]+(?P<val>.+?)\n',
            'handle': r'(NetHandle):[^\S\n]+(?P<val>.+?)\n',
            'description': r'(OrgName|CustName):[^\S\n]+(?P<val>.+?)'
                    '(?=(\n\S):?)',
            'country': r'(Country):[^\S\n]+(?P<val>.+?)\n',
            'state': r'(StateProv):[^\S\n]+(?P<val>.+?)\n',
            'city': r'(City):[^\S\n]+(?P<val>.+?)\n',
            'address': r'(Address):[^\S\n]+(?P<val>.+?)(?=(\n\S):?)',
            'postal_code': r'(PostalCode):[^\S\n]+(?P<val>.+?)\n',
            'emails': (
                r'.+?:.*?[^\S\n]+(?P<val>[\w\-\.]+?@[\w\-\.]+\.[\w\-]+)('
                '[^\S\n]+.*?)*?\n'
            ),
            'created': r'(RegDate):[^\S\n]+(?P<val>.+?)\n',
            'updated': r'(Updated):[^\S\n]+(?P<val>.+?)\n',
        },
        'dt_format': '%Y-%m-%d'
    },
    'ripencc': {
        'server': 'whois.ripe.net',
        'fields': {
            'name': r'(netname):[^\S\n]+(?P<val>.+?)\n',
            'handle': r'(nic-hdl):[^\S\n]+(?P<val>.+?)\n',
            'description': r'(descr):[^\S\n]+(?P<val>.+?)(?=(\n\S):?)',
            'country': r'(country):[^\S\n]+(?P<val>.+?)\n',
            'address': r'(address):[^\S\n]+(?P<val>.+?)(?=(\n\S):?)',
            'emails': (
                r'.+?:.*?[^\S\n]+(?P<val>[\w\-\.]+?@[\w\-\.]+\.[\w\-]+)('
                '[^\S\n]+.*?)*?\n'
            ),
            'created': (
                r'(created):[^\S\n]+(?P<val>[0-9]{4}-[0-9]{2}-[0-9]{2}T[0-9]'
                '{2}:[0-9]{2}:[0-9]{2}Z).*?\n'
            ),
            'updated': (
                r'(last-modified):[^\S\n]+(?P<val>[0-9]{4}-[0-9]{2}-[0-9]{2}T'
                '[0-9]{2}:[0-9]{2}:[0-9]{2}Z).*?\n'
            )
        },
        'dt_format': '%Y-%m-%dT%H:%M:%SZ'
    },
    'apnic': {
        'server': 'whois.apnic.net',
        'fields': {
            'name': r'(netname):[^\S\n]+(?P<val>.+?)\n',
            'handle': r'(nic-hdl):[^\S\n]+(?P<val>.+?)\n',
            'description': r'(descr):[^\S\n]+(?P<val>.+?)(?=(\n\S):?)',
            'country': r'(country):[^\S\n]+(?P<val>.+?)\n',
            'address': r'(address):[^\S\n]+(?P<val>.+?)(?=(\n\S):?)',
            'emails': (
                r'.+?:.*?[^\S\n]+(?P<val>[\w\-\.]+?@[\w\-\.]+\.[\w\-]+)('
                '[^\S\n]+.*?)*?\n'
            ),
            'updated': r'(changed):[^\S\n]+.*(?P<val>[0-9]{8}).*?\n'
        },
        'dt_format': '%Y%m%d'
    },
    'lacnic': {
        'server': 'whois.lacnic.net',
        'fields': {
            'handle': r'(nic-hdl):[^\S\n]+(?P<val>.+?)\n',
            'description': r'(owner):[^\S\n]+(?P<val>.+?)(?=(\n\S):?)',
            'country': r'(country):[^\S\n]+(?P<val>.+?)\n',
            'emails': (
                r'.+?:.*?[^\S\n]+(?P<val>[\w\-\.]+?@[\w\-\.]+\.[\w\-]+)('
                '[^\S\n]+.*?)*?\n'
            ),
            'created': r'(created):[^\S\n]+(?P<val>[0-9]{8}).*?\n',
            'updated': r'(changed):[^\S\n]+(?P<val>[0-9]{8}).*?\n'
        },
        'dt_format': '%Y%m%d'
    },
    'afrinic': {
        'server': 'whois.afrinic.net',
        'fields': {
            'name': r'(netname):[^\S\n]+(?P<val>.+?)\n',
            'handle': r'(nic-hdl):[^\S\n]+(?P<val>.+?)\n',
            'description': r'(descr):[^\S\n]+(?P<val>.+?)(?=(\n\S):?)',
            'country': r'(country):[^\S\n]+(?P<val>.+?)\n',
            'address': r'(address):[^\S\n]+(?P<val>.+?)(?=(\n\S):?)',
            'emails': (
                r'.+?:.*?[^\S\n]+(?P<val>[\w\-\.]+?@[\w\-\.]+\.[\w\-]+)('
                '[^\S\n]+.*?)*?\n'
            ),
        }
    }
}

RWHOIS = {
    'fields': {
        'cidr': r'(network:IP-Network):(?P<val>.+?)\n',
        'name': r'(network:ID):(?P<val>.+?)\n',
        'description': (
            r'(network:(Org-Name|Organization(;I)?)):(?P<val>.+?)\n'
        ),
        'country': r'(network:(Country|Country-Code)):(?P<val>.+?)\n',
        'state': r'(network:State):(?P<val>.+?)\n',
        'city': r'(network:City):(?P<val>.+?)\n',
        'address': r'(network:Street-Address):(?P<val>.+?)\n',
        'postal_code': r'(network:Postal-Code):(?P<val>.+?)\n',
        'emails': (
            r'.+?:.*?[^\S\n]+(?P<val>[\w\-\.]+?@[\w\-\.]+\.[\w\-]+)('
            '[^\S\n]+.*?)*?\n'
        ),
        'created': r'(network:Created):(?P<val>.+?)\n',
        'updated': r'(network:Updated):(?P<val>.+?)\n'
    }
}

ASN_REFERRALS = {
    'whois://whois.ripe.net': 'ripencc',
    'whois://whois.apnic.net': 'apnic',
    'whois://whois.lacnic.net': 'lacnic',
    'whois://whois.afrinic.net': 'afrinic',
}


class Whois:
    """
    The class for parsing via whois

    Args:
        net: A ipwhois.net.Net object.

    Raises:
        NetError: The parameter provided is not an instance of
            ipwhois.net.Net
        IPDefinedError: The address provided is defined (does not need to be
            resolved).
    """

    def __init__(self, net):

        from .net import Net

        # ipwhois.net.Net validation
        if isinstance(net, Net):

            self._net = net

        else:

            raise NetError('The provided net parameter is not an instance of '
                           'ipwhois.net.Net')

    def _parse_fields(self, response, fields_dict, net_start=None,
                      net_end=None, dt_format=None, field_list=None):
        """
        The function for parsing whois fields from a data input.

        Args:
            response: The response from the whois/rwhois server.
            fields_dict: The dictionary of fields -> regex search values.
            net_start: The starting point of the network (if parsing multiple
                networks).
            net_end: The ending point of the network (if parsing multiple
                networks).
            dt_format: The format of datetime fields if known.
            field_list: If provided, a list of fields to parse:
                ['name', 'handle', 'description', 'country', 'state', 'city',
                'address', 'postal_code', 'emails', 'created', 'updated']

        Returns:
            Dictionary: A dictionary of fields provided in fields_dict.
        """

        ret = {}

        if not field_list:

            field_list = ['name', 'handle', 'description', 'country', 'state',
                          'city', 'address', 'postal_code', 'emails',
                          'created', 'updated']

        generate = ((field, pattern) for (field, pattern) in
                    fields_dict.items() if field in field_list)

        for field, pattern in generate:

            pattern = re.compile(
                str(pattern),
                re.DOTALL
            )

            if net_start is not None:

                match = pattern.finditer(response, net_end, net_start)

            elif net_end is not None:

                match = pattern.finditer(response, net_end)

            else:

                match = pattern.finditer(response)

            values = []
            sub_section_end = None
            for m in match:

                if sub_section_end:

                    if field not in (
                        'emails'
                    ) and (sub_section_end != (m.start() - 1)):

                        break

                try:

                    values.append(m.group('val').strip())

                except IndexError:

                    pass

                sub_section_end = m.end()

            if len(values) > 0:

                value = None
                try:

                    if field == 'country':

                        value = values[0].upper()

                    elif field in ['created', 'updated'] and dt_format:

                        value = datetime.strptime(
                            values[0],
                            str(dt_format)).isoformat('T')

                    elif field in ['emails']:

                        value = list(unique_everseen(values))

                    else:

                        values = unique_everseen(values)
                        value = '\n'.join(values).strip()

                except ValueError as e:

                    log.debug('Whois field parsing failed for {0}: {1}'.format(
                        field, e))
                    pass

                ret[field] = value

        return ret

    def _get_nets_arin(self, response):
        """
        The function for parsing network blocks from ARIN whois data.

        Args:
            response: The response from the ARIN whois server.

        Returns:
            List: A of dictionaries containing keys: cidr, start, end.
        """

        nets = []

        # Find the first NetRange value.
        pattern = re.compile(
            r'^NetRange:[^\S\n]+(.+)$',
            re.MULTILINE
        )
        temp = pattern.search(response)
        net_range = None
        net_range_start = None
        if temp is not None:
            net_range = temp.group(1).strip()
            net_range_start = temp.start()

        # Iterate through all of the networks found, storing the CIDR value
        # and the start and end positions.
        for match in re.finditer(
            r'^CIDR:[^\S\n]+(.+?,[^\S\n].+|.+)$',
            response,
            re.MULTILINE
        ):

            try:

                net = copy.deepcopy(BASE_NET)

                if len(nets) > 0:
                    temp = pattern.search(response, match.start())
                    net_range = None
                    net_range_start = None
                    if temp is not None:
                        net_range = temp.group(1).strip()
                        net_range_start = temp.start()

                if net_range is not None:
                    if net_range_start < match.start() or len(nets) > 0:
                        net['range'] = net_range

                net['cidr'] = ', '.join(
                    [ip_network(c.strip()).__str__()
                     for c in match.group(1).split(', ')]
                )
                net['start'] = match.start()
                net['end'] = match.end()
                nets.append(net)

            except ValueError:

                pass

        return nets

    def _get_nets_lacnic(self, response):
        """
        The function for parsing network blocks from LACNIC whois data.

        Args:
            response: The response from the LACNIC whois server.

        Returns:
            List: A of dictionaries containing keys: cidr, start, end.
        """

        nets = []

        # Iterate through all of the networks found, storing the CIDR value
        # and the start and end positions.
        for match in re.finditer(
            r'^(inetnum|inet6num|route):[^\S\n]+(.+?,[^\S\n].+|.+)$',
            response,
            re.MULTILINE
        ):

            try:

                net = copy.deepcopy(BASE_NET)
                net['range'] = match.group(2).strip()

                temp = []
                for addr in match.group(2).strip().split(', '):

                    count = addr.count('.')
                    if count is not 0 and count < 4:

                        addr_split = addr.strip().split('/')
                        for i in range(count + 1, 4):
                            addr_split[0] += '.0'

                        addr = '/'.join(addr_split)

                    temp.append(ip_network(addr.strip()).__str__())

                net['cidr'] = ', '.join(temp)
                net['start'] = match.start()
                net['end'] = match.end()
                nets.append(net)

            except ValueError:

                pass

        return nets

    def _get_nets_other(self, response):
        """
        The function for parsing network blocks from generic whois data.

        Args:
            response: The response from the whois/rwhois server.

        Returns:
            List: A of dictionaries containing keys: cidr, start, end.
        """

        nets = []

        # Iterate through all of the networks found, storing the CIDR value
        # and the start and end positions.
        for match in re.finditer(
            r'^(inetnum|inet6num|route):[^\S\n]+((.+?)[^\S\n]-[^\S\n](.+)|'
                '.+)$',
            response,
            re.MULTILINE
        ):

            try:

                net = copy.deepcopy(BASE_NET)
                net['range'] = match.group(2)

                if match.group(3) and match.group(4):

                    addrs = []
                    addrs.extend(summarize_address_range(
                        ip_address(match.group(3).strip()),
                        ip_address(match.group(4).strip())))

                    cidr = ', '.join(
                        [i.__str__() for i in collapse_addresses(addrs)]
                    )

                else:

                    cidr = ip_network(match.group(2).strip()).__str__()

                net['cidr'] = cidr
                net['start'] = match.start()
                net['end'] = match.end()
                nets.append(net)

            except (ValueError, TypeError):

                pass

        return nets

    def lookup(self, inc_raw=False, retry_count=3, response=None,
               get_referral=False, extra_blacklist=None,
               ignore_referral_errors=False, asn_data=None,
               field_list=None, is_offline=False):
        """
        The function for retrieving and parsing whois information for an IP
        address via port 43/tcp (WHOIS).

        Args:
            inc_raw: Boolean for whether to include the raw results in the
                returned dictionary.
            retry_count: The number of times to retry in case socket errors,
                timeouts, connection resets, etc. are encountered.
            response: Optional response object, this bypasses the Whois lookup.
            get_referral: Boolean for whether to retrieve referral whois
                information, if available.
            extra_blacklist: A list of blacklisted whois servers in addition to
                the global BLACKLIST.
            ignore_referral_errors: Boolean for whether to ignore and continue
                when an exception is encountered on referral whois lookups.
            asn_data: Optional ASN result object, this bypasses the ASN lookup.
            field_list: If provided, a list of fields to parse:
                ['name', 'handle', 'description', 'country', 'state', 'city',
                'address', 'postal_code', 'emails', 'created', 'updated']
            is_offline: Boolean for whether to perform lookups offline. If
                True, response and asn_data must be provided. Primarily used
                for testing.

        Returns:
            Dictionary:

            :query: The IP address (String)
            :asn: The Autonomous System Number (String)
            :asn_date: The ASN Allocation date (String)
            :asn_registry: The assigned ASN registry (String)
            :asn_cidr: The assigned ASN CIDR (String)
            :asn_country_code: The assigned ASN country code (String)
            :nets: Dictionaries containing network information which consists
                of the fields listed in the NIC_WHOIS dictionary. (List)
            :raw: Raw whois results if the inc_raw parameter is True. (String)
            :referral: Dictionary of referral whois information if get_referral
                is True and the server isn't blacklisted. Consists of fields
                listed in the RWHOIS dictionary.
            :raw_referral: Raw referral whois results if the inc_raw parameter
                is True. (String)
        """

        # Create the return dictionary.
        results = {
            'query': self._net.address_str,
            'nets': [],
            'raw': None,
            'referral': None,
            'raw_referral': None
        }

        # The referral server and port. Only used if get_referral is True.
        referral_server = None
        referral_port = 0

        # Only fetch the response if we haven't already.
        if response is None or (not is_offline and
                                asn_data['asn_registry'] is not 'arin'):

            log.debug('Response not given, perform WHOIS lookup for {0}'
                      .format(self._net.address_str))

            # Retrieve the whois data.
            response = self._net.get_whois(
                asn_registry=asn_data['asn_registry'], retry_count=retry_count,
                extra_blacklist=extra_blacklist
            )

            if get_referral:

                # Search for a referral server.
                for match in re.finditer(
                    r'^ReferralServer:[^\S\n]+(.+:[0-9]+)$',
                    response,
                    re.MULTILINE
                ):

                    try:

                        temp = match.group(1)
                        if 'rwhois://' not in temp:  # pragma: no cover
                            raise ValueError

                        temp = temp.replace('rwhois://', '').split(':')

                        if int(temp[1]) > 65535:  # pragma: no cover
                            raise ValueError

                        referral_server = temp[0]
                        referral_port = int(temp[1])

                    except (ValueError, KeyError):  # pragma: no cover

                        continue

                    break

        # Retrieve the referral whois data.
        if get_referral and referral_server:

            log.debug('Perform referral WHOIS lookup')

            response_ref = None

            try:

                response_ref = self._net.get_whois(
                    asn_registry='', retry_count=retry_count,
                    server=referral_server, port=referral_port,
                    extra_blacklist=extra_blacklist
                )

            except (BlacklistError, WhoisLookupError):

                if ignore_referral_errors:

                    pass

                else:

                    raise

            if response_ref:

                log.debug('Parsing referral WHOIS data')

                if inc_raw:

                    results['raw_referral'] = response_ref

                temp_rnet = self._parse_fields(
                    response_ref,
                    RWHOIS['fields'],
                    field_list=field_list
                )

                # Add the networks to the return dictionary.
                results['referral'] = temp_rnet

        # If inc_raw parameter is True, add the response to return dictionary.
        if inc_raw:

            results['raw'] = response

        nets = []

        if asn_data['asn_registry'] == 'arin':

            nets_response = self._get_nets_arin(response)

        elif asn_data['asn_registry'] == 'lacnic':

            nets_response = self._get_nets_lacnic(response)

        else:

            nets_response = self._get_nets_other(response)

        nets.extend(nets_response)

        # Iterate through all of the network sections and parse out the
        # appropriate fields for each.
        log.debug('Parsing WHOIS data')
        for index, net in enumerate(nets):

            section_end = None
            if index + 1 < len(nets):

                section_end = nets[index + 1]['start']

            try:

                dt_format = RIR_WHOIS[results['asn_registry']]['dt_format']

            except KeyError:

                dt_format = None

            temp_net = self._parse_fields(
                response,
                RIR_WHOIS[asn_data['asn_registry']]['fields'],
                section_end,
                net['end'],
                dt_format,
                field_list
            )

            # Merge the net dictionaries.
            net.update(temp_net)

            # The start and end values are no longer needed.
            del net['start'], net['end']

        # Add the networks to the return dictionary.
        results['nets'] = nets

        return results



#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v11l4n>
  Purpose: Print Better Helper!
  Created: 2017/1/2
"""

import unittest
from colorama import init, Fore


#----------------------------------------------------------------------
def print_bar(text=None, basic_char='=', length=70, color=None):
    """"""
    if text:
        text = str(text)
        textlen = len(text)
        length_one_side = (length - textlen) / 2
        bar_side = basic_char * length_one_side
        bar = bar_side + text + bar_side
    else:
        bar = basic_char * length

    if color and hasattr(Fore, color.upper()):
        bar = getattr(Fore, color.upper()) + bar
    else:
        bar = bar
        
    print(bar)
    
########################################################################
class PrintBarTest(unittest.case.TestCase):
    """"""

    #----------------------------------------------------------------------
    def runTest(self):
        """Constructor"""
        print_bar('WellCome TO MY G3ar')
        print_bar(basic_char='0')
        print_bar(color='red')
    

if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v1ll4n>
  Purpose: Main Module for decologger
  Created: 2016/12/18
"""


import unittest
import logging
import time
import sys
import os
import traceback
import pprint
import json
from os.path import join as pathjoin
from functools import wraps
from logging.handlers import TimedRotatingFileHandler
    
    
#----------------------------------------------------------------------
def get_date():
    """"""
    return time.strftime("%Y%m%d", time.localtime(time.time()))

#----------------------------------------------------------------------
def get_func_info(func):
    """"""
    assert callable(func)
    return '[Module:' + str(__package__) + '.'\
           +str(__name__) + ' FunctionName:' + func.__name__ + ']'
    

#----------------------------------------------------------------------
def get_log_formatter(rule):
    """"""
    return logging.Formatter(rule)
    

LOW_LEVEL_LOG_PARAMS = {
    "log_called":True,
    'log_trace':False,
    "log_exception":True,
    'log_notify_admin':False
}

MIDDLE_LEVEL_LOG_PARAMS = {
    "log_called":True,
    'log_trace':False,
    "log_exception":True,
    'log_notify_admin':False
}

HIGH_LEVEL_LOG_PARAMS = {
    "log_called":True,
    'log_trace':True,
    "log_exception":True,
    'log_notify_admin':True
}

DEFAULT_FORMATTER_RULE = '%(asctime)s-%(name)s-%(levelname)s:' + \
    ' %(message)s -- [process_id:%(process)s]' + \
    '[thread_name:%(threadName)s-id:%(thread)s]'


########################################################################
class Decologger(object):
    """"""
    

    #----------------------------------------------------------------------
    def __init__(self, name, root_log_level='warning', 
                 basedir='decolog/', email_config={},):
        """Constructor
        
        Params:
            name: :str: the name of decologger
            root_log_level: :str: root logger level [debug/info/warning/error/critical]
            basedir: :str: the base path for all logs
            email_config: :str: Not Finished (If the crucial event happend, email to admin)
        """
        self._name = name
        self._email_config = email_config
        # set basedir
        self._basedir = os.path.abspath(basedir)
        self._basedir = pathjoin(self._basedir, self._name + '/')
        if not os.path.exists(self._basedir):
            os.makedirs(self._basedir)
        
        #
        # init root logger
        #
        assert isinstance(self._name, (unicode, str))
        self._root_logger = logging.getLogger(self._name)
        self._root_logger.setLevel(root_log_level.upper())
        
        root_logger_filehandler = logging.handlers.TimedRotatingFileHandler(
            filename=pathjoin(self._basedir, self._name + '.log'), 
            when='D'
        )
        root_logger_filehandler.setFormatter(get_log_formatter(DEFAULT_FORMATTER_RULE))
        
        stdouthandler = logging.StreamHandler(sys.stdout)
        stdouthandler.setFormatter(get_log_formatter(DEFAULT_FORMATTER_RULE))
        stderrhandler = logging.StreamHandler(sys.stderr)
        stderrhandler.setFormatter(get_log_formatter(DEFAULT_FORMATTER_RULE))
        
        self._root_logger.addHandler(root_logger_filehandler)
        self._root_logger.addHandler(stderrhandler)
        self._root_logger.addHandler(stdouthandler)
        
        #
        # Init sub logger
        #
        self._info = self._init_sub_logger('info')
        self._debug = self._init_sub_logger('debug')
        self._warning = self._init_sub_logger('warning')
        self._error = self._init_sub_logger('error')
        self._critical = self._init_sub_logger('critical')
        
        #
        # Init crucial logger
        #
        self._crucial_logger = self._init_sub_logger('crucial')
        
    #----------------------------------------------------------------------
    def _init_sub_logger(self, sub_logger_name):
        """"""
        level = sub_logger_name.upper()
        
        sublogger_fullname = self._name + '.' + sub_logger_name
        sublogger_filename = sublogger_fullname + '.log'
        sublogger_filename_full = pathjoin(self._basedir, sublogger_filename)
        
        filehandler = logging.handlers.TimedRotatingFileHandler(
            filename=sublogger_filename_full,
            when='D',        
        )
        filehandler.setFormatter(get_log_formatter(DEFAULT_FORMATTER_RULE))
        
        logger = logging.getLogger(sublogger_fullname)
        
        if level in ['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL']:
            logger.setLevel(level)
        elif level == 'CRUCIAL':
            logger.setLevel('CRITICAL')
        else:
            pass
        
        logger.addHandler(filehandler)
        
        if sub_logger_name.upper() == "CRITICAL" or \
           sub_logger_name.upper() == 'CRUCIAL':
            if self._email_config == {}:
                pass
            else:
                NotImplemented('[!] Under constructed')
        elif sub_logger_name.upper() == 'ERROR':
            #
            # if neccessary, define it
            #
            pass
        
        return logger
    
    #----------------------------------------------------------------------
    @property    
    def name(self):
        """"""
        return self._name
    
    #----------------------------------------------------------------------
    def low_level(self, func):
        """"""
        return self.costom(**LOW_LEVEL_LOG_PARAMS)(func)
        
    #----------------------------------------------------------------------
    def middle_level(self, func):
        """"""
        return self.costom(**MIDDLE_LEVEL_LOG_PARAMS)(func)
        
    #----------------------------------------------------------------------
    def high_level(self, func):
        """"""
        return self.costom(**HIGH_LEVEL_LOG_PARAMS)(func)
        
    
    #----------------------------------------------------------------------
    def ordinary(self, func):
        """"""
        return self.costom(**LOW_LEVEL_LOG_PARAMS)(func)

    #----------------------------------------------------------------------
    def essential(self, func):
        """"""
        return self.costom(**MIDDLE_LEVEL_LOG_PARAMS)(func)
        
    #----------------------------------------------------------------------
    def crucial(self, func):
        """"""
        #print 'function recved'
        #print 'Crucial Called'
        return self.costom(**HIGH_LEVEL_LOG_PARAMS)(func)
    
    #----------------------------------------------------------------------
    def ordinary_dctr(self, func):
        """"""
        print 'function recved'
        return func
        
    
    #----------------------------------------------------------------------
    def costom(self, log_called=True,
               log_trace=False,
               log_exception=True,
               log_notify_admin=False):
        """"""
        def _costom(func):
            @wraps(func)
            def __costom(*vargs, **kwargs):
                ret = None
                try:
                    
                    if log_trace:
                        self.debug(msg='Into: ' + get_func_info(func))
                    if log_called:
                        self.info(msg=get_func_info(func) + ' Be Called')
                        
                    ret = func(*vargs, **kwargs)
                    
                    if log_trace:
                        self.debug(msg='Out of: ' + get_func_info(func))
                except Exception, e:
                    if log_exception:
                        self.crucial_log(repr(traceback.print_exc()))
                    
                    if log_notify_admin:
                        ret = traceback.format_exc()
                        self.notify(ret)
                    raise e
                finally:
                    return ret
            return __costom
        
        return _costom
        

    #----------------------------------------------------------------------
    def notify_costom(self):
        """"""
        pass
    
    #----------------------------------------------------------------------
    def info(self, msg):
        """"""
        self._info.info(msg)
    
    #----------------------------------------------------------------------
    def debug(self, msg):
        """"""
        self._debug.debug(msg)
        
    #----------------------------------------------------------------------
    def warning(self, msg):
        """"""
        self._warning.warning(msg)
        
    #----------------------------------------------------------------------
    def error(self, msg):
        """"""
        self._error.error(msg)
    
    #----------------------------------------------------------------------
    def crucial_log(self, msg):
        """"""
        self._crucial_logger.critical(msg)
        self._critical.critical(msg)
        
    #----------------------------------------------------------------------
    def critical(self, msg):
        """"""
        self._critical.critical(msg)
        
        
    #----------------------------------------------------------------------
    def notify(self, msg, output_console=True, use_email=False):
        """"""
        if use_email:
            raise NotImplemented()
        
        if output_console:
            pprint.pprint(msg)

    

if __name__ == '__main__':
    dclogger = Decologger(name='HelloDeco')
    
    #----------------------------------------------------------------------
    @dclogger.crucial
    def test():
        """"""
        print 'testcalled'
        print 'Success'
        return "Success"
    
    print test()
    
        
    
    unittest.main()


#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v1ll4n>
  Purpose: G3ar entry
  Created: 2016/12/20
"""

import unittest

from .threadutils import thread_pool
from .threadutils import contractor
from .decologger import decologger
from .dict_parser import dict_parser
from .taskbulter import task_bulter
from .utils import ip_calc_utils
from .utils.pyping import pyping

ThreadPool = thread_pool.Pool
Contractor = contractor.Contractor
DictParser = dict_parser.DictParser
DecoLogger = decologger.Decologger
TaskBulter = task_bulter.TaskBulter
IPv4CalcUtils = ip_calc_utils

#----------------------------------------------------------------------
def ping(host, timeout=2, count=4):
    """"""
    return pyping(host, timeout, count)
    

if __name__ == '__main__':
    unittest.main()


# Copyright (c) 2013, 2014, 2015, 2016 Philip Hane
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.


class NetError(Exception):
    """
    An Exception for when a parameter provided is not an instance of
    ipwhois.net.Net.
    """


class IPDefinedError(Exception):
    """
    An Exception for when the IP is defined (does not need to be resolved).
    """


class ASNLookupError(Exception):
    """
    An Exception for when the ASN lookup failed.
    """


class ASNRegistryError(Exception):
    """
    An Exception for when the ASN registry does not match one of the five
    expected values (arin, ripencc, apnic, lacnic, afrinic).
    """


class HostLookupError(Exception):
    """
    An Exception for when the host lookup failed.
    """


class BlacklistError(Exception):
    """
    An Exception for when the server is in a blacklist.
    """


class WhoisLookupError(Exception):
    """
    An Exception for when the whois lookup failed.
    """


class HTTPLookupError(Exception):
    """
    An Exception for when the RDAP lookup failed.
    """


class HTTPRateLimitError(Exception):
    """
    An Exception for when HTTP queries exceed the NIC's request limit and have
    exhausted all retries.
    """


class InvalidEntityContactObject(Exception):
    """
    An Exception for when JSON output is not an RDAP entity contact information
    object:
    https://tools.ietf.org/html/rfc7483#section-5.4
    """


class InvalidNetworkObject(Exception):
    """
    An Exception for when JSON output is not an RDAP network object:
    https://tools.ietf.org/html/rfc7483#section-5.4
    """


class InvalidEntityObject(Exception):
    """
    An Exception for when JSON output is not an RDAP entity object:
    https://tools.ietf.org/html/rfc7483#section-5.1
    """



#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v1ll4n>
  Purpose: G3ar setup
  Created: 2016/12/20
"""

from codecs import open as copen
from setuptools import setup, find_packages

packages = find_packages()

requires = ['IPy', 'ipwhois', 'colorama']

version = '0.0.7b1'

#
# LOAD README.md
#
try:
    readme = None
    with copen('README.md', encoding='utf-8') as f:
        readme = f.read()
        
    history = None
    with copen('HISTORY.md', encoding='utf-8') as f:
        history = f.read()
except:
    readme = 'Python Coding Toolkit for Pentester.' + \
        '\nGithub: https://github.com/VillanCh/g3ar' + \
        '\nREADME.md: https://github.com/VillanCh/g3ar/blob/master/README.md\n\n'
    history = 'https://github.com/VillanCh/g3ar/blob/master/HISTORY.md'
    
    
setup(
    name='g3ar',
    version=version,
    description='Python Coding Toolkit for Pentester.',
    long_description=readme + '\n\n' + history,
    author='v1ll4n/VillanCh',
    author_email='v1ll4n@villanch.top',
    url='https://github.com/VillanCh/g3ar',
    packages=packages,
    package_data={"":["LICENSE", 'README.md', 'HISTORY.md']},
    package_dir={'g3ar':'g3ar'},
    include_package_data=True,
    install_requires=requires,
    license='BSD 2-Clause License',
    zip_safe=False,
)


# Copyright (c) 2013, 2014, 2015, 2016 Philip Hane
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from . import Net
from .nir import NIRWhois
import logging

log = logging.getLogger(__name__)


class IPWhois:
    """
    The wrapper class for performing whois/RDAP lookups and parsing for
    IPv4 and IPv6 addresses.

    Args:
        address: An IPv4 or IPv6 address as a string, integer, IPv4Address, or
            IPv6Address.
        timeout: The default timeout for socket connections in seconds.
        proxy_opener: The urllib.request.OpenerDirector request for proxy
            support or None.
        allow_permutations: allow net.Net() to use additional methods if DNS
            lookups to Cymru fail.
    """

    def __init__(self, address, timeout=5, proxy_opener=None,
                 allow_permutations=True):

        self.net = Net(
            address=address, timeout=timeout, proxy_opener=proxy_opener,
            allow_permutations=allow_permutations
        )

        self.address = self.net.address
        self.timeout = self.net.timeout
        self.address_str = self.net.address_str
        self.version = self.net.version
        self.reversed = self.net.reversed
        self.dns_zone = self.net.dns_zone

    def __repr__(self):

        return 'IPWhois({0}, {1}, {2})'.format(
            self.address_str, str(self.timeout), repr(self.net.opener)
        )

    def lookup(self, *args, **kwargs):
        """
        Temporary wrapper for legacy whois lookups (moved to
        IPWhois.lookup_whois()). This will be removed in a future
        release (1.0.0).
        """

        from warnings import warn
        warn("IPWhois.lookup() has been deprecated and will be removed. "
             "You should now use IPWhois.lookup_whois() for legacy whois "
             "lookups.")
        return self.lookup_whois(*args, **kwargs)

    def lookup_whois(self, inc_raw=False, retry_count=3, get_referral=False,
                     extra_blacklist=None, ignore_referral_errors=False,
                     field_list=None, asn_alts=None, extra_org_map=None,
                     inc_nir=True, nir_field_list=None):
        """
        The function for retrieving and parsing whois information for an IP
        address via port 43 (WHOIS).

        Args:
            inc_raw: Boolean for whether to include the raw whois results in
                the returned dictionary.
            retry_count: The number of times to retry in case socket errors,
                timeouts, connection resets, etc. are encountered.
            get_referral: Boolean for whether to retrieve referral whois
                information, if available.
            extra_blacklist: A list of blacklisted whois servers in addition to
                the global BLACKLIST.
            ignore_referral_errors: Boolean for whether to ignore and continue
                when an exception is encountered on referral whois lookups.
            field_list: If provided, a list of fields to parse:
                ['name', 'handle', 'description', 'country', 'state', 'city',
                'address', 'postal_code', 'emails', 'created', 'updated']
            asn_alts: Array of additional lookup types to attempt if the
                ASN dns lookup fails. Allow permutations must be enabled.
                Defaults to all ['whois', 'http'].
            extra_org_map: Dictionary mapping org handles to RIRs. This is for
                limited cases where ARIN REST (ASN fallback HTTP lookup) does
                not show an RIR as the org handle e.g., DNIC (which is now the
                built in ORG_MAP) e.g., {'DNIC': 'arin'}. Valid RIR values are
                (note the case-sensitive - this is meant to match the REST
                result): 'ARIN', 'RIPE', 'apnic', 'lacnic', 'afrinic'
            inc_nir: Boolean for whether to retrieve NIR (National Internet
                Registry) information, if registry is JPNIC (Japan) or KRNIC
                (Korea). If True, extra network requests will be required.
                If False, the information returned for JP or KR IPs is
                severely restricted.
            nir_field_list: If provided and inc_nir, a list of fields to parse:
                ['name', 'handle', 'country', 'address', 'postal_code',
                'nameservers', 'created', 'updated', 'contacts']

        Returns:
            Dictionary:

            :query: The IP address (String)
            :asn: The Autonomous System Number (String)
            :asn_date: The ASN Allocation date (String)
            :asn_registry: The assigned ASN registry (String)
            :asn_cidr: The assigned ASN CIDR (String)
            :asn_country_code: The assigned ASN country code (String)
            :nets: Dictionaries containing network information which consists
                of the fields listed in the ipwhois.whois.RIR_WHOIS dictionary.
                (List)
            :raw: Raw whois results if the inc_raw parameter is True. (String)
            :referral: Dictionary of referral whois information if get_referral
                is True and the server isn't blacklisted. Consists of fields
                listed in the ipwhois.whois.RWHOIS dictionary.
            :raw_referral: Raw referral whois results if the inc_raw parameter
                is True. (String)
            :nir: (Dictionary) - nir.NIRWhois() results if inc_nir is True.
        """

        from .whois import Whois

        # Create the return dictionary.
        results = {'nir': None}

        # Retrieve the ASN information.
        log.debug('ASN lookup for {0}'.format(self.address_str))
        asn_data, response = self.net.lookup_asn(
            retry_count=retry_count, asn_alts=asn_alts,
            extra_org_map=extra_org_map
        )

        # Add the ASN information to the return dictionary.
        results.update(asn_data)

        # Retrieve the whois data and parse.
        whois = Whois(self.net)
        log.debug('WHOIS lookup for {0}'.format(self.address_str))
        whois_data = whois.lookup(
            inc_raw=inc_raw, retry_count=retry_count, response=response,
            get_referral=get_referral, extra_blacklist=extra_blacklist,
            ignore_referral_errors=ignore_referral_errors, asn_data=asn_data,
            field_list=field_list
        )

        # Add the WHOIS information to the return dictionary.
        results.update(whois_data)

        if inc_nir:

            nir = None
            if 'JP' == asn_data['asn_country_code']:
                nir = 'jpnic'
            elif 'KR' == asn_data['asn_country_code']:
                nir = 'krnic'

            if nir:

                nir_whois = NIRWhois(self.net)
                nir_data = nir_whois.lookup(
                    nir=nir, inc_raw=inc_raw, retry_count=retry_count,
                    response=None,
                    field_list=nir_field_list, is_offline=False
                )

                # Add the NIR information to the return dictionary.
                results['nir'] = nir_data

        return results

    def lookup_rdap(self, inc_raw=False, retry_count=3, depth=0,
                    excluded_entities=None, bootstrap=False,
                    rate_limit_timeout=120, asn_alts=None, extra_org_map=None,
                    inc_nir=True, nir_field_list=None):
        """
        The function for retrieving and parsing whois information for an IP
        address via HTTP (RDAP).

        **This is now the recommended method, as RDAP contains much better
        information to parse.**

        Args:
            inc_raw: Boolean for whether to include the raw whois results in
                the returned dictionary.
            retry_count: The number of times to retry in case socket errors,
                timeouts, connection resets, etc. are encountered.
            depth: How many levels deep to run queries when additional
                referenced objects are found.
            excluded_entities: A list of entity handles to not perform lookups.
            bootstrap: If True, performs lookups via ARIN bootstrap rather
                than lookups based on ASN data. ASN lookups are not performed
                and no output for any of the asn* fields is provided.
            rate_limit_timeout: The number of seconds to wait before retrying
                when a rate limit notice is returned via rdap+json.
            asn_alts: Array of additional lookup types to attempt if the
                ASN dns lookup fails. Allow permutations must be enabled.
                Defaults to all ['whois', 'http'].
            extra_org_map: Dictionary mapping org handles to RIRs. This is for
                limited cases where ARIN REST (ASN fallback HTTP lookup) does
                not show an RIR as the org handle e.g., DNIC (which is now the
                built in ORG_MAP) e.g., {'DNIC': 'arin'}. Valid RIR values are
                (note the case-sensitive - this is meant to match the REST
                result): 'ARIN', 'RIPE', 'apnic', 'lacnic', 'afrinic'
            inc_nir: Boolean for whether to retrieve NIR (National Internet
                Registry) information, if registry is JPNIC (Japan) or KRNIC
                (Korea). If True, extra network requests will be required.
                If False, the information returned for JP or KR IPs is
                severely restricted.
            nir_field_list: If provided and inc_nir, a list of fields to parse:
                ['name', 'handle', 'country', 'address', 'postal_code',
                'nameservers', 'created', 'updated', 'contacts']

        Returns:
            Dictionary:

            :query: The IP address (String)
            :asn: The Autonomous System Number (String)
            :asn_date: The ASN Allocation date (String)
            :asn_registry: The assigned ASN registry (String)
            :asn_cidr: The assigned ASN CIDR (String)
            :asn_country_code: The assigned ASN country code (String)
            :entities: List of entity handles referred by the top level query.
            :network: Dictionary containing network information which consists
                of the fields listed in the ipwhois.rdap._RDAPNetwork dict.
            :objects: Dictionary of (entity handle: entity dict) which consists
                of the fields listed in the ipwhois.rdap._RDAPEntity dict.
            :raw: (Dictionary) - Whois results in json format if the inc_raw
                parameter is True.
            :nir: (Dictionary) - nir.NIRWhois() results if inc_nir is True.
        """

        from .rdap import RDAP

        # Create the return dictionary.
        results = {'nir': None}

        asn_data = None
        response = None
        if not bootstrap:

            # Retrieve the ASN information.
            log.debug('ASN lookup for {0}'.format(self.address_str))
            asn_data, asn_response = self.net.lookup_asn(
                retry_count=retry_count, asn_alts=asn_alts,
                extra_org_map=extra_org_map
            )

            # Add the ASN information to the return dictionary.
            results.update(asn_data)

        # Retrieve the RDAP data and parse.
        rdap = RDAP(self.net)
        log.debug('RDAP lookup for {0}'.format(self.address_str))
        rdap_data = rdap.lookup(
            inc_raw=inc_raw, retry_count=retry_count, asn_data=asn_data,
            depth=depth, excluded_entities=excluded_entities,
            response=response, bootstrap=bootstrap,
            rate_limit_timeout=rate_limit_timeout
        )

        # Add the RDAP information to the return dictionary.
        results.update(rdap_data)

        if inc_nir:

            nir = None
            if 'JP' == asn_data['asn_country_code']:
                nir = 'jpnic'
            elif 'KR' == asn_data['asn_country_code']:
                nir = 'krnic'

            if nir:
                nir_whois = NIRWhois(self.net)
                nir_data = nir_whois.lookup(
                    nir=nir, inc_raw=inc_raw, retry_count=retry_count,
                    response=None,
                    field_list=nir_field_list, is_offline=False
                )

                # Add the NIR information to the return dictionary.
                results['nir'] = nir_data

        return results



#!/usr/bin/env python
#coding:utf-8
"""
  Author:   --<v1ll4n>
  Purpose: Simple and Quick Parse your dictionary FOR TENTEST PROGRAMMER!
  Created: 2016/12/16
"""

import dict_parser
from dict_parser import *

def get_dictparser(filename, 
                   session_id=DEFAULT_SESSION_ID, 
                   do_continue=False,
                   session_data_file=SESSION_TABLE_FILE):
    return DictParser(filename, session_id, do_continue, session_data_file)


    


# Copyright (c) 2013, 2014, 2015, 2016 Philip Hane
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from . import NetError
from .utils import unique_everseen
import logging
import sys
import re
import copy
from datetime import (datetime, timedelta)

if sys.version_info >= (3, 3):  # pragma: no cover
    from ipaddress import (ip_address,
                           ip_network,
                           summarize_address_range,
                           collapse_addresses)
else:  # pragma: no cover
    from ipaddr import (IPAddress as ip_address,
                        IPNetwork as ip_network,
                        summarize_address_range,
                        collapse_address_list as collapse_addresses)

log = logging.getLogger(__name__)

# Base NIR whois output dictionary.
BASE_NET = {
    'cidr': None,
    'name': None,
    'handle': None,
    'range': None,
    'country': None,
    'address': None,
    'postal_code': None,
    'nameservers': None,
    'created': None,
    'updated': None,
    'contacts': None
}

# Base NIR whois contact output dictionary.
BASE_CONTACT = {
    'name': None,
    'email': None,
    'reply_email': None,
    'organization': None,
    'division': None,
    'title': None,
    'phone': None,
    'fax': None,
    'updated': None
}

# National Internet Registry
NIR_WHOIS = {
    'jpnic': {
        'country_code': 'JP',
        'url': ('http://whois.nic.ad.jp/cgi-bin/whois_gw?lang=%2Fe&key={0}'
                '&submit=query'),
        'request_type': 'GET',
        'request_headers': {'Accept': 'text/html'},
        'form_data_ip_field': None,
        'fields': {
            'name': r'(\[Organization\])[^\S\n]+(?P<val>.*?)\n',
            'handle': r'(\[Network Name\])[^\S\n]+(?P<val>.*?)\n',
            'created': r'(\[Assigned Date\])[^\S\n]+(?P<val>.*?)\n',
            'updated': r'(\[Last Update\])[^\S\n]+(?P<val>.*?)\n',
            'nameservers': r'(\[Nameserver\])[^\S\n]+(?P<val>.*?)\n',
            'contact_admin': r'(\[Administrative Contact\])[^\S\n]+.+?\>'
                             '(?P<val>.+?)\<\/A\>\n',
            'contact_tech': r'(\[Technical Contact\])[^\S\n]+.+?\>'
                             '(?P<val>.+?)\<\/A\>\n'
        },
        'contact_fields': {
            'name': r'(\[Last, First\])[^\S\n]+(?P<val>.*?)\n',
            'email': r'(\[E-Mail\])[^\S\n]+(?P<val>.*?)\n',
            'reply_email': r'(\[Reply Mail\])[^\S\n]+(?P<val>.*?)\n',
            'organization': r'(\[Organization\])[^\S\n]+(?P<val>.*?)\n',
            'division': r'(\[Division\])[^\S\n]+(?P<val>.*?)\n',
            'title': r'(\[Title\])[^\S\n]+(?P<val>.*?)\n',
            'phone': r'(\[TEL\])[^\S\n]+(?P<val>.*?)\n',
            'fax': r'(\[FAX\])[^\S\n]+(?P<val>.*?)\n',
            'updated': r'(\[Last Update\])[^\S\n]+(?P<val>.*?)\n'
        },
        'dt_format': '%Y/%m/%d %H:%M:%S(JST)',
        'dt_hourdelta': 9,
        'multi_net': False
    },
    'krnic': {
        'country_code': 'KR',
        'url': 'http://whois.kisa.or.kr/eng/whois.jsc',
        'request_type': 'POST',
        'request_headers': {'Accept': 'text/html'},
        'form_data_ip_field': 'query',
        'fields': {
            'name': r'(Organization Name)[\s]+\:[^\S\n]+(?P<val>.+?)\n',
            'handle': r'(Service Name|Network Type)[\s]+\:[^\S\n]+(?P<val>.+?)'
                      '\n',
            'address': r'(Address)[\s]+\:[^\S\n]+(?P<val>.+?)\n',
            'postal_code': r'(Zip Code)[\s]+\:[^\S\n]+(?P<val>.+?)\n',
            'created': r'(Registration Date)[\s]+\:[^\S\n]+(?P<val>.+?)\n',
            'contact_admin': r'(id="eng_isp_contact").+?\>(?P<val>.*?)\<'
                              '\/div\>\n',
            'contact_tech': r'(id="eng_user_contact").+?\>(?P<val>.*?)\<'
                             '\/div\>\n'
        },
        'contact_fields': {
            'name': r'(Name)[^\S\n]+?:[^\S\n]+?(?P<val>.*?)\n',
            'email': r'(E-Mail)[^\S\n]+?:[^\S\n]+?(?P<val>.*?)\n',
            'phone': r'(Phone)[^\S\n]+?:[^\S\n]+?(?P<val>.*?)\n'
        },
        'dt_format': '%Y%m%d',
        'dt_hourdelta': 0,
        'multi_net': True
    }
}


class NIRWhois:
    """
    The class for parsing whois data for NIRs (National Internet Registry).
    JPNIC and KRNIC are currently the only NIRs supported. Output varies
    based on NIR specific whois formatting.

    Args:
        net: A ipwhois.net.Net object.

    Raises:
        NetError: The parameter provided is not an instance of
            ipwhois.net.Net
        IPDefinedError: The address provided is defined (does not need to be
            resolved).
    """

    def __init__(self, net):

        from .net import Net

        # ipwhois.net.Net validation
        if isinstance(net, Net):

            self._net = net

        else:

            raise NetError('The provided net parameter is not an instance of '
                           'ipwhois.net.Net')

    def _parse_fields(self, response, fields_dict, net_start=None,
                      net_end=None, dt_format=None, field_list=None,
                      hourdelta=0, is_contact=False):
        """
        The function for parsing whois fields from a data input.

        Args:
            response: The response from the whois/rwhois server.
            fields_dict: The dictionary of fields -> regex search values.
            net_start: The starting point of the network (if parsing multiple
                networks).
            net_end: The ending point of the network (if parsing multiple
                networks).
            dt_format: The format of datetime fields if known.
            field_list: If provided, a list of fields to parse:
                ['name', 'handle', 'country', 'address', 'postal_code',
                'nameservers', 'created', 'updated', 'contacts']
            is_contact: If True, uses contact information field parsing.

        Returns:
            Dictionary: A dictionary of fields provided in fields_dict.
        """

        response = '{0}\n'.format(response)
        if is_contact:

            ret = {}

            if not field_list:

                field_list = list(BASE_CONTACT.keys())

        else:

            ret = {
                'contacts': {'admin': None, 'tech': None},
                'contact_admin': {},
                'contact_tech': {}
            }

            if not field_list:

                field_list = list(BASE_NET.keys())
                field_list.remove('contacts')
                field_list.append('contact_admin')
                field_list.append('contact_tech')

        generate = ((field, pattern) for (field, pattern) in
                    fields_dict.items() if field in field_list)

        for field, pattern in generate:

            pattern = re.compile(
                str(pattern),
                re.DOTALL
            )

            if net_start is not None:

                match = pattern.finditer(response, net_end, net_start)

            elif net_end is not None:

                match = pattern.finditer(response, net_end)

            else:

                match = pattern.finditer(response)

            values = []
            for m in match:

                try:

                    values.append(m.group('val').strip())

                except IndexError:

                    pass

            if len(values) > 0:

                value = None
                try:

                    if field in ['created', 'updated'] and dt_format:

                        value = (
                            datetime.strptime(
                                values[0],
                                str(dt_format)
                            ) - timedelta(hours=hourdelta)
                        ).isoformat('T')

                    elif field in ['nameservers']:

                        value = list(unique_everseen(values))

                    else:

                        values = unique_everseen(values)
                        value = '\n'.join(values)

                except ValueError as e:

                    log.debug('NIR whois field parsing failed for {0}: {1}'
                              ''.format(field, e))
                    pass

                ret[field] = value

        return ret

    def _get_nets_jpnic(self, response):
        """
        The function for parsing network blocks from jpnic whois data.

        Args:
            response: The response from the jpnic server.

        Returns:
            List: A of dictionaries containing keys: cidr, start, end.
        """

        nets = []

        # Iterate through all of the networks found, storing the CIDR value
        # and the start and end positions.
        for match in re.finditer(
                r'^.*?(\[Network Number\])[^\S\n]+.+?>(?P<val>.+?)</A>$',
                response,
                re.MULTILINE
        ):

            try:

                net = copy.deepcopy(BASE_NET)
                tmp = ip_network(match.group(2))

                try:  # pragma: no cover
                    network_address = tmp.network_address
                except AttributeError:  # pragma: no cover
                    network_address = tmp.ip
                    pass

                try:  # pragma: no cover
                    broadcast_address = tmp.broadcast_address
                except AttributeError:  # pragma: no cover
                    broadcast_address = tmp.broadcast
                    pass

                net['range'] = '{0} - {1}'.format(
                    network_address + 1, broadcast_address
                )

                cidr = ip_network(match.group(2).strip()).__str__()

                net['cidr'] = cidr
                net['start'] = match.start()
                net['end'] = match.end()
                nets.append(net)

            except (ValueError, TypeError):

                pass

        return nets

    def _get_nets_krnic(self, response):
        """
        The function for parsing network blocks from krnic whois data.

        Args:
            response: The response from the krnic server.

        Returns:
            List: A of dictionaries containing keys: cidr, start, end.
        """

        nets = []

        # Iterate through all of the networks found, storing the CIDR value
        # and the start and end positions.
        for match in re.finditer(
                r'^(IPv4 Address)[\s]+:[^\S\n]+((.+?)[^\S\n]-[^\S\n](.+?)'
                '[^\S\n]\((.+?)\)|.+)$',
                response,
                re.MULTILINE
        ):

            try:

                net = copy.deepcopy(BASE_NET)
                net['range'] = match.group(2)

                if match.group(3) and match.group(4):

                    addrs = []
                    addrs.extend(summarize_address_range(
                        ip_address(match.group(3).strip()),
                        ip_address(match.group(4).strip())))

                    cidr = ', '.join(
                        [i.__str__() for i in collapse_addresses(addrs)]
                    )

                    net['range'] = '{0} - {1}'.format(
                        match.group(3), match.group(4)
                    )

                else:

                    cidr = ip_network(match.group(2).strip()).__str__()

                net['cidr'] = cidr
                net['start'] = match.start()
                net['end'] = match.end()
                nets.append(net)

            except (ValueError, TypeError):

                pass

        return nets

    def _get_contact(self, response=None, nir=None, handle=None,
                     retry_count=None, dt_format=None):
        """
        The function for retrieving and parsing NIR whois data based on
        NIR_WHOIS contact_fields.

        Args:
            response: Optional response object, this bypasses the lookup.
            nir: The NIR to query ('jpnic' or 'krnic').
            handle: For NIRs that have seperate contact queries (JPNIC),
                this is the contact handle to use in the query.
            retry_count: The number of times to retry in case socket errors,
                timeouts, connection resets, etc. are encountered.
            dt_format: The format of datetime fields if known.

        Returns:
            Dictionary: A dictionary of fields provided in contact_fields.
        """

        if response or nir == 'krnic':

            contact_response = response

        else:

            # Retrieve the whois data.
            contact_response = self._net.get_http_raw(
                url=str(NIR_WHOIS[nir]['url']).format(handle),
                retry_count=retry_count,
                headers=NIR_WHOIS[nir]['request_headers'],
                request_type=NIR_WHOIS[nir]['request_type']
            )

        return self._parse_fields(
            response=contact_response,
            fields_dict=NIR_WHOIS[nir]['contact_fields'],
            dt_format=dt_format,
            hourdelta=int(NIR_WHOIS[nir]['dt_hourdelta']),
            is_contact=True
        )

    def lookup(self, nir=None, inc_raw=False, retry_count=3, response=None,
               field_list=None, is_offline=False):
        """
        The function for retrieving and parsing NIR whois information for an IP
        address via HTTP (HTML scraping).

        Args:
            nir: The NIR to query ('jpnic' or 'krnic').
            inc_raw: Boolean for whether to include the raw results in the
                returned dictionary.
            retry_count: The number of times to retry in case socket errors,
                timeouts, connection resets, etc. are encountered.
            response: Optional response object, this bypasses the NIR lookup.
            field_list: If provided, a list of fields to parse:
                ['name', 'handle', 'country', 'address', 'postal_code',
                'nameservers', 'created', 'updated', 'contacts']
            is_offline: Boolean for whether to perform lookups offline. If
                True, response and asn_data must be provided. Primarily used
                for testing.

        Returns:
            Dictionary:

            :query: The IP address (String)
            :nets: List of dictionaries containing network information which
                consists of the fields listed in the NIR_WHOIS dictionary.
            :raw: Raw NIR whois results if the inc_raw parameter is True.
                (String)
        """

        if nir not in NIR_WHOIS.keys():

            raise KeyError('Invalid arg for nir (National Internet Registry')

        # Create the return dictionary.
        results = {
            'query': self._net.address_str,
            'raw': None
        }

        # Only fetch the response if we haven't already.
        if response is None:

            if is_offline:

                raise KeyError('response argument required when '
                               'is_offline=True')

            log.debug('Response not given, perform WHOIS lookup for {0}'
                      .format(self._net.address_str))

            form_data = None
            if NIR_WHOIS[nir]['form_data_ip_field']:
                form_data = {NIR_WHOIS[nir]['form_data_ip_field']:
                             self._net.address_str}

            # Retrieve the whois data.
            response = self._net.get_http_raw(
                url=str(NIR_WHOIS[nir]['url']).format(self._net.address_str),
                retry_count=retry_count,
                headers=NIR_WHOIS[nir]['request_headers'],
                request_type=NIR_WHOIS[nir]['request_type'],
                form_data=form_data
            )

        # If inc_raw parameter is True, add the response to return dictionary.
        if inc_raw:

            results['raw'] = response

        nets = []
        nets_response = None
        if nir == 'jpnic':

            nets_response = self._get_nets_jpnic(response)

        elif nir == 'krnic':

            nets_response = self._get_nets_krnic(response)

        nets.extend(nets_response)

        global_contacts = {}

        # Iterate through all of the network sections and parse out the
        # appropriate fields for each.
        log.debug('Parsing NIR WHOIS data')
        for index, net in enumerate(nets):

            section_end = None
            if index + 1 < len(nets):
                section_end = nets[index + 1]['start']

            try:

                dt_format = NIR_WHOIS[nir]['dt_format']

            except KeyError:  # pragma: no cover

                dt_format = None

            temp_net = self._parse_fields(
                response=response,
                fields_dict=NIR_WHOIS[nir]['fields'],
                net_start=section_end,
                net_end=net['end'],
                dt_format=dt_format,
                field_list=field_list,
                hourdelta=int(NIR_WHOIS[nir]['dt_hourdelta'])
            )
            temp_net['country'] = NIR_WHOIS[nir]['country_code']
            contacts = {
                'admin': temp_net['contact_admin'],
                'tech': temp_net['contact_tech']
            }

            del (
                temp_net['contact_admin'],
                temp_net['contact_tech']
            )

            if not is_offline:

                for key, val in contacts.items():

                    if len(val) > 0:

                        if isinstance(val, str):

                            val = [val]

                        for contact in val:

                            if contact in global_contacts.keys():

                                temp_net['contacts'][key] = (
                                    global_contacts[contact]
                                )

                            else:

                                if nir == 'krnic':

                                    tmp_response = contact
                                    tmp_handle = None

                                else:

                                    tmp_response = None
                                    tmp_handle = contact

                                temp_net['contacts'][key] = self._get_contact(
                                    response=tmp_response,
                                    handle=tmp_handle,
                                    nir=nir,
                                    retry_count=retry_count,
                                    dt_format=dt_format
                                )
                                global_contacts[contact] = (
                                    temp_net['contacts'][key]
                                )

            # Merge the net dictionaries.
            net.update(temp_net)

            # The start and end values are no longer needed.
            del net['start'], net['end']

        # Add the networks to the return dictionary.
        results['nets'] = nets

        return results
